//! Asymmetric key handling and wrappers

use ::num_traits::FromPrimitive;
use ::std::vec::Vec;

use super::Error;

/// Public key ID
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct KeyID(pub u16);

/// Kind of key in the handshake
#[derive(Clone)]
#[allow(missing_debug_implementations)]
pub enum PrivKey {
    /// Keys to be used only in key exchanges, not for signing
    Exchange(ExchangePrivKey),
    /// Keys to be used only for signing
    Signing,
}

/// Ephemeral private keys
#[derive(Clone)]
#[allow(missing_debug_implementations)]
pub enum ExchangePrivKey {
    /// X25519(Curve25519) used for key exchange
    X25519(::x25519_dalek::StaticSecret),
}

impl ExchangePrivKey {
    /// Run the key exchange between two keys of the same kind
    pub fn key_exchange(
        &self,
        pub_key: ExchangePubKey,
    ) -> Result<[u8; 32], Error> {
        todo!();
    }
}

/// Kind of key in the key exchange
#[derive(Debug, Copy, Clone, ::num_derive::FromPrimitive)]
#[repr(u8)]
pub enum ExchangePubKeyKind {
    /// X25519 Public key
    X25519 = 0,
}
impl ExchangePubKeyKind {
    fn len(&self) -> usize {
        match self {
            // FIXME: 99% wrong size
            ExchangePubKeyKind::X25519 => {
                ::ring::signature::ED25519_PUBLIC_KEY_LEN
            }
        }
    }
}

/// all Ephemeral Public keys
#[derive(Debug, Copy, Clone)]
pub enum ExchangePubKey {
    /// X25519(Curve25519) used for key exchange
    X25519(::x25519_dalek::PublicKey),
}

impl ExchangePubKey {
    /// Load public key used for key exchange from it raw bytes
    /// The riesult is "unparsed" since we don't verify
    /// the actual key
    pub fn from_slice(raw: &[u8]) -> Result<(Self, usize), Error> {
        // FIXME: get *real* minimum key size
        const MIN_KEY_SIZE: usize = 8;
        if raw.len() < 1 + MIN_KEY_SIZE {
            return Err(Error::NotEnoughData);
        }
        match ExchangePubKeyKind::from_u8(raw[0]) {
            Some(kind) => match kind {
                ExchangePubKeyKind::X25519 => {
                    let pub_key: ::x25519_dalek::PublicKey =
                        match ::bincode::deserialize(&raw[1..(1 + kind.len())])
                        {
                            Ok(pub_key) => pub_key,
                            Err(_) => return Err(Error::Parsing),
                        };
                    Ok((ExchangePubKey::X25519(pub_key), kind.len()))
                }
            },
            None => {
                return Err(Error::Parsing);
            }
        }
    }
}
